import { JSX, Accessor } from 'solid-js';
import { Narrow } from '@solid-primitives/utils';

type TokenElement<T> = (() => JSX.Element) & {
    data: T;
};
declare const TYPE: unique symbol;
type JSXParser<T> = {
    readonly id: symbol;
    [TYPE]: T;
};
type JSXParserData<T extends JSXParser<any>> = T[typeof TYPE];
/**
 * Creates a JSX Parser that can be used to create tokenized components and parse JSX Elements for tokens.
 *
 * @param options - Additional options for the parser
 * @param options.name - The name of the parser, used for debugging
 * @returns parser object required by `createToken`, `childrenTokens` and `isToken` functions.
 * @example
 * ```tsx
 * const parser = createJSXParser();
 *
 * const MyToken = createToken(parser, props => ({ type: "my-token", props }));
 *
 * const MyComponent = props => {
 *   const tokens = resolveTokens(parser, () => props.children);
 *   return <ul>
 *     <For each={tokens()}>{token => <li>{token.data.type}</li>}</For>
 *   </ul>
 * }
 *
 * <MyComponent>
 *   <MyToken/>
 *   <MyToken/>
 * </MyComponent>
 * ```
 */
declare function createJSXParser<T>(options?: {
    name: string;
}): JSXParser<T>;
/**
 * Creates a token component associated with the corresponding jsx-parser.
 * @param parser object returned by `createJSXParser`
 * @param tokenData function that returns the data of the token
 * @param render function that returns the fallback JSX Element to render
 * @returns a token component
 */
declare function createToken<P extends {
    [key: string]: any;
}, T extends Tokens, Tokens>(parser: JSXParser<Tokens>, tokenData: (props: P) => Narrow<T>, render?: (props: P) => JSX.Element): (props: P) => TokenElement<T>;
declare function createToken<T>(parser: JSXParser<T>, tokenData?: undefined, render?: (props: T) => JSX.Element): (props: T) => TokenElement<T>;
/**
 * Resolves all tokens in a JSX Element
 * @param parser object returned by `createJSXParser`
 * @param fn accessor that returns a JSX Element
 * @returns accessor that returns an array of tokens
 */
declare function resolveTokens<T>(parser: JSXParser<T>, fn: Accessor<JSX.Element>): Accessor<TokenElement<T>[]>;
/**
 * Resolves all tokens in a JSX Element and returns the data of the tokens
 * @param parser object returned by `createJSXParser`
 * @param fn accessor that returns a JSX Element
 * @returns accessor that returns an array of token data
 */
declare function resolveData<T>(parser: JSXParser<T>, fn: Accessor<JSX.Element>): Accessor<T[]>;
/**
 * Checks if a value is a token
 * @param parser object returned by `createJSXParser`
 * @param value value to check
 * @returns true if value is a token
 */
declare function isToken<T>(parser: JSXParser<T>, value: unknown | TokenElement<T>): value is TokenElement<T>;

export { JSXParser, JSXParserData, TokenElement, createJSXParser, createToken, isToken, resolveData, resolveTokens };
